==> syncfusion_flutter_gauges-18.2.45.tar.gz: lib/src/radial_gauge/gauge_painter/radial_axis_painter.dart <==
part of gauges;

/// Custom painter to paint gauge axis
class _AxisPainter extends CustomPainter {
  _AxisPainter(
      this._gauge,
      this._axis,
      this._isRepaint,
      ValueNotifier<num> notifier,
      this._axisLineAnimation,
      this._axisElementsAnimation,
      this._gaugeThemeData,
      this._animationDetails)
      : super(repaint: notifier);

  /// Specifies the circular gauge
  final SfRadialGauge _gauge;

  /// Specifies whether to repaint the series
  final bool _isRepaint;

  /// Specifies the axis of the painter
  final RadialAxis _axis;

  /// Specifies the animation for axis line
  final Animation<double> _axisLineAnimation;

  /// Specifies the animation for axis elements
  final Animation<double> _axisElementsAnimation;

  /// Specifies the gauge theme data.
  final SfGaugeThemeData _gaugeThemeData;

  /// Hold the radial gauge animation details
  final _AnimationDetails _animationDetails;

  @override
  void paint(Canvas canvas, Size size) {
    if (_axis.backgroundImage != null &&
        _axis._backgroundImageInfo?.image != null) {
      double _radius;
      Rect _rect;
      if (!_axis.canScaleToFit) {
        _radius = math.min(_axis._axisSize.width, _axis._axisSize.height) / 2;
        _rect = Rect.fromLTRB(
            _axis._axisSize.width / 2 - _radius - _axis._centerX,
            _axis._axisSize.height / 2 - _radius - _axis._centerY,
            _axis._axisSize.width / 2 + _radius - _axis._centerX,
            _axis._axisSize.height / 2 + _radius - _axis._centerY);
      } else {
        _radius = _axis._radius;
        _rect = Rect.fromLTRB(
            _axis._axisCenter.dx - _radius,
            _axis._axisCenter.dy - _radius,
            _axis._axisCenter.dx + _radius,
            _axis._axisCenter.dx + _radius);
      }

      // Draws the background image of axis
      paintImage(
        canvas: canvas,
        rect: _rect,
        scale: _axis._backgroundImageInfo.scale ?? 1,
        image: _axis._backgroundImageInfo.image,
        fit: BoxFit.fill,
      );
    }

    if (_axis.showAxisLine && _axis._actualAxisWidth > 0) {
      _drawAxisLine(canvas);
    }

    if (_axis.showTicks) {
      _drawMajorTicks(canvas);
      _drawMinorTicks(canvas);
    }

    if (_axis.showLabels) {
      _drawAxisLabels(canvas);
    }

    if (_gauge.axes[_gauge.axes.length - 1] == _axis &&
        _animationDetails._needsToAnimateAxes &&
        _hasAxisLineAnimation() &&
        _hasAxisElementsAnimation()) {
      _animationDetails._needsToAnimateAxes = false;
    }
  }

  /// Checks whether the show axis line is enabled
  bool _hasAxisLineAnimation() {
    return (_axisLineAnimation != null && _axisLineAnimation.value == 1) ||
        _axisLineAnimation == null;
  }

  /// Checks whether the labels and the ticks are enabled
  bool _hasAxisElementsAnimation() {
    return (_axisElementsAnimation != null &&
            _axisElementsAnimation.value == 1) ||
        _axisElementsAnimation == null;
  }

  /// Method to draw the axis line
  void _drawAxisLine(Canvas canvas) {
    // whether the dash array is enabled for axis.
    final bool isDashedAxisLine = _isDashedLine();
    SweepGradient gradient;
    if (_axis.axisLineStyle.gradient != null &&
        _axis.axisLineStyle.gradient.colors != null &&
        _axis.axisLineStyle.gradient.colors.isNotEmpty) {
      gradient = SweepGradient(
          stops: _calculateGradientStops(
              _getGradientOffset(), _axis.isInversed, _axis._sweepAngle),
          colors: _axis.isInversed
              ? _axis.axisLineStyle.gradient.colors.reversed.toList()
              : _axis.axisLineStyle.gradient.colors);
    }
    if (_axis.axisLineStyle.cornerStyle == CornerStyle.bothFlat ||
        isDashedAxisLine) {
      _drawAxisPath(canvas, _axis._startRadian, _axis._endRadian, gradient,
          isDashedAxisLine);
    } else {
      _drawAxisPath(canvas, _axis._startCornerRadian, _axis._sweepCornerRadian,
          gradient, isDashedAxisLine);
    }
  }

  /// Returns the gradient stop of axis line gradient
  List<double> _getGradientOffset() {
    if (_axis.axisLineStyle.gradient.stops != null &&
        _axis.axisLineStyle.gradient.stops.isNotEmpty) {
      return _axis.axisLineStyle.gradient.stops;
    } else {
      // Calculates the gradient stop values based on the provided color
      final double _difference = 1 / _axis.axisLineStyle.gradient.colors.length;
      final List<double> _offsets =
          List<double>(_axis.axisLineStyle.gradient.colors.length);
      for (int i = 0; i < _axis.axisLineStyle.gradient.colors.length; i++) {
        _offsets[i] = i * _difference;
      }

      return _offsets;
    }
  }

  /// Method to draw axis line
  void _drawAxisPath(Canvas canvas, double _startRadian, double _endRadian,
      SweepGradient gradient, bool isDashedAxisLine) {
    if (_axisLineAnimation != null) {
      _endRadian = _endRadian * _axisLineAnimation.value;
    }

    canvas.save();
    canvas.translate(_axis._axisCenter.dx, _axis._axisCenter.dy);
    canvas.rotate(_axis.isInversed
        ? _degreeToRadian(_axis.startAngle + _axis._sweepAngle)
        : _degreeToRadian(_axis.startAngle));

    Path path = Path();
    //whether the style of paint is fill
    bool isFill = false;
    if (_axis.axisLineStyle.cornerStyle != CornerStyle.bothFlat) {
      if (isDashedAxisLine) {
        path = _getPath(_endRadian, isFill);
      } else {
        isFill = true;
        final double _outerRadius = _axis._radius - _axis._axisOffset;
        final double _innerRadius = _outerRadius - _axis._actualAxisWidth;

        // Adds the rounded corner at start of axis line
        if (_axis.axisLineStyle.cornerStyle == CornerStyle.startCurve ||
            _axis.axisLineStyle.cornerStyle == CornerStyle.bothCurve) {
          _drawStartCurve(path, _endRadian, _innerRadius, _outerRadius);
        }

        path.addArc(
            Rect.fromCircle(center: const Offset(0, 0), radius: _outerRadius),
            _axis._startCornerRadian,
            _endRadian);

        // Adds the rounded corner at end of axis line
        if (_axis.axisLineStyle.cornerStyle == CornerStyle.endCurve ||
            _axis.axisLineStyle.cornerStyle == CornerStyle.bothCurve) {
          _drawEndCurve(path, _endRadian, _innerRadius, _outerRadius);
        }
        path.arcTo(
            Rect.fromCircle(center: const Offset(0, 0), radius: _innerRadius),
            _endRadian + _axis._startCornerRadian,
            -_endRadian,
            false);
      }
    } else {
      path = _getPath(_endRadian, isFill);
    }

    final Paint _paint = _getPaint(gradient, isFill);
    if (!isDashedAxisLine) {
      canvas.drawPath(path, _paint);
    } else {
      canvas.drawPath(
          _dashPath(path,
              dashArray:
                  _CircularIntervalList<double>(_axis.axisLineStyle.dashArray)),
          _paint);
    }

    canvas.restore();
  }

  /// Returns the axis path
  Path _getPath(double _endRadian, bool isFill) {
    final Path _path = Path();
    isFill = false;
    if (_axis.isInversed) {
      _endRadian = _endRadian * -1;
    }

    _path.addArc(_axis._axisRect, 0, _endRadian);
    return _path;
  }

  Paint _getPaint(SweepGradient gradient, bool isFill) {
    final Paint _paint = Paint()
      ..color = _axis.axisLineStyle.color ?? _gaugeThemeData.axisLineColor
      ..style = !isFill ? PaintingStyle.stroke : PaintingStyle.fill
      ..strokeWidth = _axis._actualAxisWidth;
    if (gradient != null) {
      _paint.shader = gradient.createShader(_axis._axisRect);
    }

    return _paint;
  }

  /// Draws the start corner style
  void _drawStartCurve(
      Path path, double _endRadian, double _innerRadius, double _outerRadius) {
    final Offset midPoint = _degreeToPoint(
        _axis.isInversed ? -_axis._cornerAngle : _axis._cornerAngle,
        (_innerRadius + _outerRadius) / 2,
        const Offset(0, 0));
    final double midStartAngle = _degreeToRadian(180);

    double midEndAngle = midStartAngle + _degreeToRadian(180);
    midEndAngle = _axis.isInversed ? -midEndAngle : midEndAngle;
    path.addArc(
        Rect.fromCircle(
            center: midPoint, radius: (_innerRadius - _outerRadius).abs() / 2),
        midStartAngle,
        midEndAngle);
  }

  ///Draws the end corner curve
  void _drawEndCurve(Path path, double _sweepRadian, double _innerRadius,
      double _outerRadius) {
    final double _cornerAngle =
        _axis.axisLineStyle.cornerStyle == CornerStyle.bothCurve
            ? _axis._cornerAngle
            : 0;
    final double _angle = _axis.isInversed
        ? _radianToDegree(_sweepRadian) - _cornerAngle
        : _radianToDegree(_sweepRadian) + _cornerAngle;
    final Offset midPoint = _degreeToPoint(
        _angle, (_innerRadius + _outerRadius) / 2, const Offset(0, 0));

    final double midStartAngle = _sweepRadian / 2;

    final double midEndAngle = _axis.isInversed
        ? midStartAngle - _degreeToRadian(180)
        : midStartAngle + _degreeToRadian(180);

    path.arcTo(
        Rect.fromCircle(
            center: midPoint, radius: (_innerRadius - _outerRadius).abs() / 2),
        midStartAngle,
        midEndAngle,
        false);
  }

  /// Checks whether the axis line is dashed line
  bool _isDashedLine() {
    return _axis.axisLineStyle.dashArray != null &&
        _axis.axisLineStyle.dashArray.isNotEmpty &&
        _axis.axisLineStyle.dashArray.length > 1 &&
        _axis.axisLineStyle.dashArray[0] > 0 &&
        _axis.axisLineStyle.dashArray[1] > 0;
  }

  /// Method to draw the major ticks
  void _drawMajorTicks(Canvas canvas) {
    double _length = _axis._majorTickOffsets.length.toDouble();
    if (_axisElementsAnimation != null) {
      _length = _axis._majorTickOffsets.length * _axisElementsAnimation.value;
    }

    if (_axis._actualMajorTickLength > 0 &&
        _axis.majorTickStyle.thickness != null &&
        _axis.majorTickStyle.thickness > 0) {
      final Paint _tickPaint = Paint()
        ..style = PaintingStyle.stroke
        ..strokeWidth = _axis.majorTickStyle.thickness;
      for (num i = 0; i < _length; i++) {
        final _TickOffset _tickOffset = _axis._majorTickOffsets[i];
        if (!(i == 0 && _axis._sweepAngle == 360)) {
          _tickPaint.color = _axis.useRangeColorForAxis
              ? _axis._getRangeColor(_tickOffset.value, _gaugeThemeData) ??
                  _axis.majorTickStyle.color ??
                  _gaugeThemeData.majorTickColor
              : _axis.majorTickStyle.color ?? _gaugeThemeData.majorTickColor;

          if (_axis.majorTickStyle.dashArray != null &&
              _axis.majorTickStyle.dashArray.isNotEmpty) {
            final Path path = Path()
              ..moveTo(_tickOffset.startPoint.dx, _tickOffset.startPoint.dy)
              ..lineTo(_tickOffset.endPoint.dx, _tickOffset.endPoint.dy);
            canvas.drawPath(
                _dashPath(path,
                    dashArray: _CircularIntervalList<double>(
                        _axis.majorTickStyle.dashArray)),
                _tickPaint);
          } else {
            if ((i == _axis._majorTickOffsets.length - 1) &&
                _axis._sweepAngle == 360) {
              // Reposition the last tick when its sweep angle is 360
              final double _x1 = (_axis._majorTickOffsets[0].startPoint.dx +
                      _axis._majorTickOffsets[i].startPoint.dx) /
                  2;
              final double _y1 = (_axis._majorTickOffsets[0].startPoint.dy +
                      _axis._majorTickOffsets[i].startPoint.dy) /
                  2;
              final double _x2 = (_axis._majorTickOffsets[0].endPoint.dx +
                      _axis._majorTickOffsets[i].endPoint.dx) /
                  2;
              final double _y2 = (_axis._majorTickOffsets[0].endPoint.dy +
                      _axis._majorTickOffsets[i].endPoint.dy) /
                  2;
              canvas.drawLine(Offset(_x1, _y1), Offset(_x2, _y2), _tickPaint);
            } else {
              canvas.drawLine(
                  _tickOffset.startPoint, _tickOffset.endPoint, _tickPaint);
            }
          }
        }
      }
    }
  }

  /// Method to draw the mior ticks
  void _drawMinorTicks(Canvas canvas) {
    double _length = _axis._minorTickOffsets.length.toDouble();
    if (_axisElementsAnimation != null) {
      _length = _axis._minorTickOffsets.length * _axisElementsAnimation.value;
    }
    if (_axis._actualMinorTickLength > 0 &&
        _axis.minorTickStyle.thickness != null &&
        _axis.minorTickStyle.thickness > 0) {
      final Paint _tickPaint = Paint()
        ..style = PaintingStyle.stroke
        ..strokeWidth = _axis.minorTickStyle.thickness;
      for (int i = 0; i < _length; i++) {
        final _TickOffset _tickOffset = _axis._minorTickOffsets[i];
        _tickPaint.color = _axis.useRangeColorForAxis
            ? _axis._getRangeColor(_tickOffset.value, _gaugeThemeData) ??
                _axis.minorTickStyle.color ??
                _gaugeThemeData.minorTickColor
            : _axis.minorTickStyle.color ?? _gaugeThemeData.minorTickColor;
        if (_axis.minorTickStyle.dashArray != null &&
            _axis.minorTickStyle.dashArray.isNotEmpty) {
          final Path path = Path()
            ..moveTo(_tickOffset.startPoint.dx, _tickOffset.startPoint.dy)
            ..lineTo(_tickOffset.endPoint.dx, _tickOffset.endPoint.dy);
          canvas.drawPath(
              _dashPath(path,
                  dashArray: _CircularIntervalList<double>(
                      _axis.minorTickStyle.dashArray)),
              _tickPaint);
        } else {
          canvas.drawLine(
              _tickOffset.startPoint, _tickOffset.endPoint, _tickPaint);
        }
      }
    }
  }

  /// Method to draw the axis labels
  void _drawAxisLabels(Canvas canvas) {
    double _length = _axis._axisLabels.length.toDouble();
    if (_axisElementsAnimation != null) {
      _length = _axis._axisLabels.length * _axisElementsAnimation.value;
    }
    for (int i = 0; i < _length; i++) {
      if (!((i == 0 && !_axis.showFirstLabel) ||
          (i == _axis._axisLabels.length - 1 && !_axis.showLastLabel))) {
        final CircularAxisLabel label = _axis._axisLabels[i];
        final Color _labelColor =
            label.labelStyle.color ?? _gaugeThemeData.axisLabelColor;
        final TextSpan span = TextSpan(
            text: label.text,
            style: TextStyle(
                color: _axis.ranges != null &&
                        _axis.ranges.isNotEmpty &&
                        _axis.useRangeColorForAxis
                    ? _axis._getRangeColor(label.value, _gaugeThemeData) ??
                        _labelColor
                    : _labelColor,
                fontSize: label.labelStyle.fontSize,
                fontFamily: label.labelStyle.fontFamily,
                fontStyle: label.labelStyle.fontStyle,
                fontWeight: label.labelStyle.fontWeight));

        final TextPainter textPainter = TextPainter(
            text: span,
            textDirection: TextDirection.ltr,
            textAlign: TextAlign.center);
        textPainter.layout();

        if (_axis.needsRotateLabels || label._needsRotateLabel) {
          canvas.save();
          canvas.translate(label.position.dx, label.position.dy);
          // Rotates the labels to its calculated angle
          canvas.rotate(_degreeToRadian(label.angle));
          canvas.scale(-1);
          textPainter.paint(canvas,
              Offset(-label.labelSize.width / 2, -label.labelSize.height / 2));
          canvas.restore();
        } else {
          textPainter.paint(
              canvas,
              Offset(label.position.dx - label.labelSize.width / 2,
                  label.position.dy - label.labelSize.height / 2));
        }
      }
    }
  }

  @override
  bool shouldRepaint(_AxisPainter oldDelegate) => _isRepaint;
}

==> syncfusion_flutter_gauges-18.2.46.tar.gz: lib/src/radial_gauge/gauge_painter/radial_axis_painter.dart <==
part of gauges;

/// Custom painter to paint gauge axis
class _AxisPainter extends CustomPainter {
  _AxisPainter(
      this._gauge,
      this._axis,
      this._isRepaint,
      ValueNotifier<num> notifier,
      this._axisLineAnimation,
      this._axisElementsAnimation,
      this._gaugeThemeData,
      this._animationDetails)
      : super(repaint: notifier);

  /// Specifies the circular gauge
  final SfRadialGauge _gauge;

  /// Specifies whether to repaint the series
  final bool _isRepaint;

  /// Specifies the axis of the painter
  final RadialAxis _axis;

  /// Specifies the animation for axis line
  final Animation<double> _axisLineAnimation;

  /// Specifies the animation for axis elements
  final Animation<double> _axisElementsAnimation;

  /// Specifies the gauge theme data.
  final SfGaugeThemeData _gaugeThemeData;

  /// Hold the radial gauge animation details
  final _AnimationDetails _animationDetails;

  @override
  void paint(Canvas canvas, Size size) {
    if (_axis.backgroundImage != null &&
        _axis._backgroundImageInfo?.image != null) {
      double radius;
      Rect rect;
      if (!_axis.canScaleToFit) {
        radius = math.min(_axis._axisSize.width, _axis._axisSize.height) / 2;
        rect = Rect.fromLTRB(
            _axis._axisSize.width / 2 - radius - _axis._centerX,
            _axis._axisSize.height / 2 - radius - _axis._centerY,
            _axis._axisSize.width / 2 + radius - _axis._centerX,
            _axis._axisSize.height / 2 + radius - _axis._centerY);
      } else {
        radius = _axis._radius;
        rect = Rect.fromLTRB(
            _axis._axisCenter.dx - radius,
            _axis._axisCenter.dy - radius,
            _axis._axisCenter.dx + radius,
            _axis._axisCenter.dx + radius);
      }

      // Draws the background image of axis
      paintImage(
        canvas: canvas,
        rect: rect,
        scale: _axis._backgroundImageInfo.scale ?? 1,
        image: _axis._backgroundImageInfo.image,
        fit: BoxFit.fill,
      );
    }

    if (_axis.showAxisLine && _axis._actualAxisWidth > 0) {
      _drawAxisLine(canvas);
    }

    if (_axis.showTicks) {
      _drawMajorTicks(canvas);
      _drawMinorTicks(canvas);
    }

    if (_axis.showLabels) {
      _drawAxisLabels(canvas);
    }

    if (_gauge.axes[_gauge.axes.length - 1] == _axis &&
        _animationDetails.needsToAnimateAxes &&
        _getHasAxisLineAnimation() &&
        _getHasAxisElementsAnimation()) {
      _animationDetails.needsToAnimateAxes = false;
    }
  }

  /// Checks whether the show axis line is enabled
  bool _getHasAxisLineAnimation() {
    return (_axisLineAnimation != null && _axisLineAnimation.value == 1) ||
        _axisLineAnimation == null;
  }

  /// Checks whether the labels and the ticks are enabled
  bool _getHasAxisElementsAnimation() {
    return (_axisElementsAnimation != null &&
            _axisElementsAnimation.value == 1) ||
        _axisElementsAnimation == null;
  }

  /// Method to draw the axis line
  void _drawAxisLine(Canvas canvas) {
    // whether the dash array is enabled for axis.
    final bool isDashedAxisLine = _getIsDashedLine();
    SweepGradient gradient;
    if (_axis.axisLineStyle.gradient != null &&
        _axis.axisLineStyle.gradient.colors != null &&
        _axis.axisLineStyle.gradient.colors.isNotEmpty) {
      gradient = SweepGradient(
          stops: _calculateGradientStops(
              _getGradientOffset(), _axis.isInversed, _axis._sweepAngle),
          colors: _axis.isInversed
              ? _axis.axisLineStyle.gradient.colors.reversed.toList()
              : _axis.axisLineStyle.gradient.colors);
    }
    if (_axis.axisLineStyle.cornerStyle == CornerStyle.bothFlat ||
        isDashedAxisLine) {
      _drawAxisPath(canvas, _axis._startRadian, _axis._endRadian, gradient,
          isDashedAxisLine);
    } else {
      _drawAxisPath(canvas, _axis._startCornerRadian, _axis._sweepCornerRadian,
          gradient, isDashedAxisLine);
    }
  }

  /// Returns the gradient stop of axis line gradient
  List<double> _getGradientOffset() {
    if (_axis.axisLineStyle.gradient.stops != null &&
        _axis.axisLineStyle.gradient.stops.isNotEmpty) {
      return _axis.axisLineStyle.gradient.stops;
    } else {
      // Calculates the gradient stop values based on the provided color
      final double difference = 1 / _axis.axisLineStyle.gradient.colors.length;
      final List<double> offsets =
          List<double>(_axis.axisLineStyle.gradient.colors.length);
      for (int i = 0; i < _axis.axisLineStyle.gradient.colors.length; i++) {
        offsets[i] = i * difference;
      }

      return offsets;
    }
  }

  /// Method to draw axis line
  void _drawAxisPath(Canvas canvas, double startRadian, double endRadian,
      SweepGradient gradient, bool isDashedAxisLine) {
    if (_axisLineAnimation != null) {
      endRadian = endRadian * _axisLineAnimation.value;
    }

    canvas.save();
    canvas.translate(_axis._axisCenter.dx, _axis._axisCenter.dy);
    canvas.rotate(_axis.isInversed
        ? _getDegreeToRadian(_axis.startAngle + _axis._sweepAngle)
        : _getDegreeToRadian(_axis.startAngle));

    Path path = Path();
    //whether the style of paint is fill
    bool isFill = false;
    if (_axis.axisLineStyle.cornerStyle != CornerStyle.bothFlat) {
      if (isDashedAxisLine) {
        path = _getPath(endRadian, isFill);
      } else {
        isFill = true;
        final double _outerRadius = _axis._radius - _axis._axisOffset;
        final double _innerRadius = _outerRadius - _axis._actualAxisWidth;

        // Adds the rounded corner at start of axis line
        if (_axis.axisLineStyle.cornerStyle == CornerStyle.startCurve ||
            _axis.axisLineStyle.cornerStyle == CornerStyle.bothCurve) {
          _drawStartCurve(path, endRadian, _innerRadius, _outerRadius);
        }

        path.addArc(
            Rect.fromCircle(center: const Offset(0, 0), radius: _outerRadius),
            _axis._startCornerRadian,
            endRadian);

        // Adds the rounded corner at end of axis line
        if (_axis.axisLineStyle.cornerStyle == CornerStyle.endCurve ||
            _axis.axisLineStyle.cornerStyle == CornerStyle.bothCurve) {
          _drawEndCurve(path, endRadian, _innerRadius, _outerRadius);
        }
        path.arcTo(
            Rect.fromCircle(center: const Offset(0, 0), radius: _innerRadius),
            endRadian + _axis._startCornerRadian,
            -endRadian,
            false);
      }
    } else {
      path = _getPath(endRadian, isFill);
    }

    _renderPath(isDashedAxisLine, path, canvas, gradient, isFill);
  }

  // Method to render the path
  void _renderPath(bool isDashedAxisLine, Path path, Canvas canvas,
      Gradient gradient, bool isFill) {
    final Paint paint = _getPaint(gradient, isFill);
    if (!isDashedAxisLine) {
      canvas.drawPath(path, paint);
    } else {
      canvas.drawPath(
          _dashPath(path,
              dashArray:
                  _CircularIntervalList<double>(_axis.axisLineStyle.dashArray)),
          paint);
    }

    canvas.restore();
  }

  /// Returns the axis path
  Path _getPath(double endRadian, bool isFill) {
    final Path _path = Path();
    isFill = false;
    if (_axis.isInversed) {
      endRadian = endRadian * -1;
    }

    _path.addArc(_axis._axisRect, 0, endRadian);
    return _path;
  }

  Paint _getPaint(SweepGradient gradient, bool isFill) {
    final Paint _paint = Paint()
      ..color = _axis.axisLineStyle.color ?? _gaugeThemeData.axisLineColor
      ..style = !isFill ? PaintingStyle.stroke : PaintingStyle.fill
      ..strokeWidth = _axis._actualAxisWidth;
    if (gradient != null) {
      _paint.shader = gradient.createShader(_axis._axisRect);
    }

    return _paint;
  }

  /// Draws the start corner style
  void _drawStartCurve(
      Path path, double endRadian, double innerRadius, double outerRadius) {
    final Offset midPoint = _getDegreeToPoint(
        _axis.isInversed ? -_axis._cornerAngle : _axis._cornerAngle,
        (innerRadius + outerRadius) / 2,
        const Offset(0, 0));
    final double midStartAngle = _getDegreeToRadian(180);

    double midEndAngle = midStartAngle + _getDegreeToRadian(180);
    midEndAngle = _axis.isInversed ? -midEndAngle : midEndAngle;
    path.addArc(
        Rect.fromCircle(
            center: midPoint, radius: (innerRadius - outerRadius).abs() / 2),
        midStartAngle,
        midEndAngle);
  }

  ///Draws the end corner curve
  void _drawEndCurve(
      Path path, double sweepRadian, double innerRadius, double outerRadius) {
    final double cornerAngle =
        _axis.axisLineStyle.cornerStyle == CornerStyle.bothCurve
            ? _axis._cornerAngle
            : 0;
    final double angle = _axis.isInversed
        ? _getRadianToDegree(sweepRadian) - cornerAngle
        : _getRadianToDegree(sweepRadian) + cornerAngle;
    final Offset midPoint = _getDegreeToPoint(
        angle, (innerRadius + outerRadius) / 2, const Offset(0, 0));

    final double midStartAngle = sweepRadian / 2;

    final double midEndAngle = _axis.isInversed
        ? midStartAngle - _getDegreeToRadian(180)
        : midStartAngle + _getDegreeToRadian(180);

    path.arcTo(
        Rect.fromCircle(
            center: midPoint, radius: (innerRadius - outerRadius).abs() / 2),
        midStartAngle,
        midEndAngle,
        false);
  }

  /// Checks whether the axis line is dashed line
  bool _getIsDashedLine() {
    return _axis.axisLineStyle.dashArray != null &&
        _axis.axisLineStyle.dashArray.isNotEmpty &&
        _axis.axisLineStyle.dashArray.length > 1 &&
        _axis.axisLineStyle.dashArray[0] > 0 &&
        _axis.axisLineStyle.dashArray[1] > 0;
  }

  /// Method to draw the major ticks
  void _drawMajorTicks(Canvas canvas) {
    double length = _axis._majorTickOffsets.length.toDouble();
    if (_axisElementsAnimation != null) {
      length = _axis._majorTickOffsets.length * _axisElementsAnimation.value;
    }

    if (_axis._actualMajorTickLength > 0 &&
        _axis.majorTickStyle.thickness != null &&
        _axis.majorTickStyle.thickness > 0) {
      final Paint tickPaint = Paint()
        ..style = PaintingStyle.stroke
        ..strokeWidth = _axis.majorTickStyle.thickness;
      for (num i = 0; i < length; i++) {
        final _TickOffset tickOffset = _axis._majorTickOffsets[i];
        if (!(i == 0 && _axis._sweepAngle == 360)) {
          tickPaint.color = _axis.useRangeColorForAxis
              ? _axis._getRangeColor(tickOffset.value, _gaugeThemeData) ??
                  _axis.majorTickStyle.color ??
                  _gaugeThemeData.majorTickColor
              : _axis.majorTickStyle.color ?? _gaugeThemeData.majorTickColor;

          if (_axis.majorTickStyle.dashArray != null &&
              _axis.majorTickStyle.dashArray.isNotEmpty) {
            final Path path = Path()
              ..moveTo(tickOffset.startPoint.dx, tickOffset.startPoint.dy)
              ..lineTo(tickOffset.endPoint.dx, tickOffset.endPoint.dy);
            canvas.drawPath(
                _dashPath(path,
                    dashArray: _CircularIntervalList<double>(
                        _axis.majorTickStyle.dashArray)),
                tickPaint);
          } else {
            if ((i == _axis._majorTickOffsets.length - 1) &&
                _axis._sweepAngle == 360) {
              // Reposition the last tick when its sweep angle is 360
              final double x1 = (_axis._majorTickOffsets[0].startPoint.dx +
                      _axis._majorTickOffsets[i].startPoint.dx) /
                  2;
              final double y1 = (_axis._majorTickOffsets[0].startPoint.dy +
                      _axis._majorTickOffsets[i].startPoint.dy) /
                  2;
              final double x2 = (_axis._majorTickOffsets[0].endPoint.dx +
                      _axis._majorTickOffsets[i].endPoint.dx) /
                  2;
              final double y2 = (_axis._majorTickOffsets[0].endPoint.dy +
                      _axis._majorTickOffsets[i].endPoint.dy) /
                  2;
              canvas.drawLine(Offset(x1, y1), Offset(x2, y2), tickPaint);
            } else {
              canvas.drawLine(
                  tickOffset.startPoint, tickOffset.endPoint, tickPaint);
            }
          }
        }
      }
    }
  }

  /// Method to draw the mior ticks
  void _drawMinorTicks(Canvas canvas) {
    double length = _axis._minorTickOffsets.length.toDouble();
    if (_axisElementsAnimation != null) {
      length = _axis._minorTickOffsets.length * _axisElementsAnimation.value;
    }
    if (_axis._actualMinorTickLength > 0 &&
        _axis.minorTickStyle.thickness != null &&
        _axis.minorTickStyle.thickness > 0) {
      final Paint tickPaint = Paint()
        ..style = PaintingStyle.stroke
        ..strokeWidth = _axis.minorTickStyle.thickness;
      for (int i = 0; i < length; i++) {
        final _TickOffset tickOffset = _axis._minorTickOffsets[i];
        tickPaint.color = _axis.useRangeColorForAxis
            ? _axis._getRangeColor(tickOffset.value, _gaugeThemeData) ??
                _axis.minorTickStyle.color ??
                _gaugeThemeData.minorTickColor
            : _axis.minorTickStyle.color ?? _gaugeThemeData.minorTickColor;
        if (_axis.minorTickStyle.dashArray != null &&
            _axis.minorTickStyle.dashArray.isNotEmpty) {
          final Path path = Path()
            ..moveTo(tickOffset.startPoint.dx, tickOffset.startPoint.dy)
            ..lineTo(tickOffset.endPoint.dx, tickOffset.endPoint.dy);
          canvas.drawPath(
              _dashPath(path,
                  dashArray: _CircularIntervalList<double>(
                      _axis.minorTickStyle.dashArray)),
              tickPaint);
        } else {
          canvas.drawLine(
              tickOffset.startPoint, tickOffset.endPoint, tickPaint);
        }
      }
    }
  }

  /// Method to draw the axis labels
  void _drawAxisLabels(Canvas canvas) {
    double length = _axis._axisLabels.length.toDouble();
    if (_axisElementsAnimation != null) {
      length = _axis._axisLabels.length * _axisElementsAnimation.value;
    }
    for (int i = 0; i < length; i++) {
      if (!((i == 0 && !_axis.showFirstLabel) ||
          (i == _axis._axisLabels.length - 1 &&
              !_axis.showLastLabel &&
              _axis._isMaxiumValueIncluded))) {
        final CircularAxisLabel label = _axis._axisLabels[i];
        final Color labelColor =
            label.labelStyle.color ?? _gaugeThemeData.axisLabelColor;
        final TextSpan span = TextSpan(
            text: label.text,
            style: TextStyle(
                color: _axis.ranges != null &&
                        _axis.ranges.isNotEmpty &&
                        _axis.useRangeColorForAxis
                    ? _axis._getRangeColor(label.value, _gaugeThemeData) ??
                        labelColor
                    : labelColor,
                fontSize: label.labelStyle.fontSize,
                fontFamily: label.labelStyle.fontFamily,
                fontStyle: label.labelStyle.fontStyle,
                fontWeight: label.labelStyle.fontWeight));

        final TextPainter textPainter = TextPainter(
            text: span,
            textDirection: TextDirection.ltr,
            textAlign: TextAlign.center);
        textPainter.layout();
        _renderText(canvas, textPainter, label);
      }
    }
  }

  // Methods to render the range label
  void _renderText(
      Canvas canvas, TextPainter textPainter, CircularAxisLabel label) {
    if (_axis.needsRotateLabels || label._needsRotateLabel) {
      canvas.save();
      canvas.translate(label.position.dx, label.position.dy);
      // Rotates the labels to its calculated angle
      canvas.rotate(_getDegreeToRadian(label.angle));
      canvas.scale(-1);
      textPainter.paint(canvas,
          Offset(-label.labelSize.width / 2, -label.labelSize.height / 2));
      canvas.restore();
    } else {
      textPainter.paint(
          canvas,
          Offset(label.position.dx - label.labelSize.width / 2,
              label.position.dy - label.labelSize.height / 2));
    }
  }

  @override
  bool shouldRepaint(_AxisPainter oldDelegate) => _isRepaint;
}
